#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Graph::Maker::TwinAlternateAreaTree;

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'TAVertexToZ_xy';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 7;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script $level\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

sub n_is_even_length {
  my ($n) = @_;
  return $n == 0 || length(sprintf '%b', $n) % 2 == 0;
}

my $even_segments = MyLineSegments->new;
my $odd_segments = MyLineSegments->new;
my $graph = Graph::Maker->new ('twin_alternate_area_tree',
                               level => $level,
                               undirected => 1,
                              );
foreach my $edge ($graph->edges) {
  my ($u,$v) = @$edge;
  my $segments = (n_is_even_length($u) && n_is_even_length($v)
                  ? $even_segments : $odd_segments);
  $segments->add(TAVertexToZ_xy($u), TAVertexToZ_xy($v));
}

$Printwrap = 2;
printwrap("\\draw[my even] ");
$Printwrap += 2;
$even_segments->printwrap;
printwrap(";\n");

$Printwrap = 2;
printwrap("\\draw[my odd] ");
$Printwrap += 2;
$odd_segments->printwrap;
printwrap(";\n");

print "  % end generated\n";
exit 0;
