#!/usr/bin/perl -w

# Copyright 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.010;
use strict;
use warnings;
use Math::PlanePath::AlternatePaper;
use Tie::IxHash;

use FindBin;
use lib $FindBin::Bin;
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 4;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script $level\n";
my $path = Math::PlanePath::AlternatePaper->new;

print "  \\begin{scope}[shift=(pos-$level)]\n";

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

my $segments = MyLineSegments->new;
tie my %nodes, 'Tie::IxHash';
tie my %matches, 'Tie::IxHash';
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  $segments->add($x,$y, $x2,$y2);
  $nodes{"$x,$y"} = 1;
  $nodes{"$x2,$y2"} = 1;
  if ($x==$y && $x2 == $x+1) {
    $matches{"$x,$y"} = 1;
  }
}

foreach my $xy (keys %matches) {
  my ($x,$y) = split /,/, $xy;
  my $x2 = $x+1;
  delete $nodes{$xy};
  delete $nodes{"$x2,$y"};
  $segments->remove($x,$y, $x2,$y);
}

$Printwrap = 4;
printwrap("\\draw[my other] ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

$Printwrap = 4;
printwrap("\\path foreach \\p in {");
$Printwrap += 2;
foreach my $xy (keys %nodes) {
  printwrap("($xy),");
}
printwrap("}");
printwrap(" { \\p node[my dot] };\n");

$Printwrap = 4;
printwrap("\\path foreach \\p in {");
$Printwrap += 2;
foreach my $xy (keys %matches) {
  printwrap("($xy),");
}
printwrap("}");
printwrap(" { \\p pic [transform shape] {my match pic} };\n");

print "  \\end{scope}\n";
print "  % end generated\n";

exit 0;
