#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use warnings;
use Math::PlanePath::AlternatePaper 124; # v.124 for n_to_n_list()

use FindBin;
use lib $FindBin::Bin;
use MyAlternate
  'xy_to_node_name', '@dir4_to_dx','@dir4_to_dy';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 7;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script $level\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
my $path = Math::PlanePath::AlternatePaper->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

$Printwrap = 2;
printwrap("\\node (start) at (0,0) [my dot];\n");
printwrap("\\node (end)   at ($x_hi,$y_hi) [my dot];\n");

my %xy_to_node = ('0,0' => 'start',
                  "$x_hi,$y_hi" => 'end');
my %seen_diff;
printwrap("\\begin{scope}[my node]\n");
foreach my $n ($n_lo .. $n_hi-1) {
  my @n_list = $path->n_to_n_list($n);
  @n_list = grep {$_ < $n_hi} @n_list;
  if (@n_list == 2) {
    my $diff = abs($n_list[0] - $n_list[1]);
    if (! $seen_diff{$diff}++) {
      my ($x,$y) = $path->n_to_xy($n);
      my $node = xy_to_node_name($x,$y);
      $xy_to_node{"$x,$y"} = $node;
      printwrap(" \\node ($node) at ($x,$y) {$diff};");
    }
  }
}
printwrap_newline();
printwrap("\\end{scope}\n");

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  $x1 *= 3;  $x2 *= 3;
  $y1 *= 3;  $y2 *= 3;
  $segments->add($x1,$y1, $x2,$y2);
}
my %sxy_to_node;
while (my ($xy, $node) = each %xy_to_node) {
  my ($x,$y) = split /,/, $xy;
  $x *= 3; $y *= 3;
  $sxy_to_node{"$x,$y"} = $node;
  foreach my $dir4 (0 .. 3) {
    my $dx = $dir4_to_dx[$dir4];
    my $dy = $dir4_to_dy[$dir4];
    my $x2 = $x + $dx;
    my $y2 = $y + $dy;
    $segments->remove($x,$y, $x2,$y2);
    $sxy_to_node{"$x2,$y2"} = $node;
  }
}
### %sxy_to_node

$Printwrap = 2;
printwrap("\\draw[my line] ");
$Printwrap += 2;
$segments->printwrap (stringize => sub {
                        my ($x,$y) = @_;
                        my $str = $sxy_to_node{"$x,$y"} // ($x/3).','.($y/3);
                        return "($str)";
                      });
printwrap(";\n");

print "  % end generated\n";
exit 0;
