#!/usr/bin/perl -w

# Copyright 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.010;
use strict;
use warnings;
use Math::NumSeq::PlanePathTurn;
use Math::PlanePath::AlternatePaper 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'xy_to_node_name';
use MyPrintwrap ':all';
use MyLineSegments;


my $x_min = -7; my $x_max = 7;
my $y_min = -3; my $y_max = 3;
my $xy_mod = 8;

print "  % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::AlternatePaper->new;
my $seq = Math::NumSeq::PlanePathTurn->new(planepath_object => $path,
                                           turn_type => 'LSR');
my %xy_to_turn;
foreach (1 .. 1024) {
  my ($n,$turn) = $seq->next;
  next unless $n % 2 == 1 || $n % 4 == 2 || $n % 8 == 4;
  my ($x,$y) = $path->n_to_xy($n);
  my $str = ($turn == 1 ? 'L'
             : $turn == -1 ? 'R'
             : die);
  my $key = ($x % $xy_mod).','.($y % $xy_mod);
  if (defined $xy_to_turn{$key}) {
    $xy_to_turn{$key} eq $str or die;
  }
  $xy_to_turn{$key} = $str;
}

$Printwrap = 2;
foreach my $y ($y_min .. $y_max) {
  foreach my $x ($x_min .. $x_max) {
    next if $x==0 && $y==0;
    my $key = ($x % $xy_mod).','.($y % $xy_mod);
    my $str = $xy_to_turn{$key};
    if (! defined $str) {
      printwrap(" \\node at ($x,$y) [my dot];");
    } elsif ($str eq 'R') {
      printwrap(" \\node at ($x,$y) [my R] {$str};");
    } else {
      printwrap(" \\node at ($x,$y) [my L] {$str};");
    }
  }
}
printwrap_newline();

# printwrap("\\draw[my line]");
# $Printwrap += 2;
# foreach my $y ($y_min .. $y_max) {  # horizontals
#   my $from = "$x_min-\\MyEnd,$y";
#   foreach my $x ($x_min .. $x_max) {
#     my $to = xy_to_node_name($x,$y);
#     if (($x+$y)%2==1) {
#       printwrap(" ($from)--($to)");
#       $from = $to;
#     }
#   }
#   my $to = "$x_max+\\MyEnd,$y";
#   printwrap(" ($from)--($to)");
# }
# foreach my $x ($x_min .. $x_max) {  # verticals
#   my $from = "$x,$y_min-\\MyEnd";
#   foreach my $y ($y_min .. $y_max) {
#     my $to = xy_to_node_name($x,$y);
#     if (($x+$y)%2==1) {
#       printwrap(" ($from)--($to)");
#       $from = $to;
#     }
#   }
#   my $to = "$x,$y_max+\\MyEnd";
#   printwrap(" ($from)--($to)");
# }
# printwrap(";\n");

print "  % end generated\n";
exit 0;
